/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvControlBinop.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_1Az0Nmhw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_W0Ux218B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PLjaVkg8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bouqE1WI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0Pe2ot7O_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_8qB8Eupi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IiEDA5Km_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_hrcymlJt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_XNqboByk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WDTI8RnF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Y4WhAFZv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_wsJgvmcF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_fJTAaFCh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_1L8cXclq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BcGm3QTU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_h6W4dZ2r_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_U0v6D5s7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3ymh8PM6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IqPKgDHx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gm0Qcw5l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3dGNsVAQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qq0w2ycR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_KqeHJvz6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_EZm0i9JO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_4wZOiaGz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_9MWdYhz8_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_gjDI1NMl;
  SignalLine sLine_0PG7Iia6;
  SignalLine sLine_juZLuNU4;
  ControlBinop cBinop_1Az0Nmhw;
  ControlBinop cBinop_W0Ux218B;
  ControlBinop cBinop_PLjaVkg8;
  ControlBinop cBinop_bouqE1WI;
  ControlBinop cBinop_IiEDA5Km;
  ControlBinop cBinop_XNqboByk;
  ControlBinop cBinop_WDTI8RnF;
  ControlBinop cBinop_Y4WhAFZv;
  ControlVar cVar_wsJgvmcF;
  ControlVar cVar_fJTAaFCh;
  SignalVarf sVarf_tYz7D1fr;
  SignalVarf sVarf_LZwTNEAn;
  SignalVarf sVarf_JFs0OP7O;
  SignalVarf sVarf_YXJTlQB5;
  ControlBinop cBinop_3ymh8PM6;
  ControlBinop cBinop_qq0w2ycR;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
